@@ -11,8 +11,8 @@ use crate::CldrPathsAllInOne;
 use std::path::PathBuf;
 
 /// Implementation of CldrPaths that downloads CLDR data directories on demand.
-/// The download artifacts are saved in the user's cache directory; see
-/// https://docs.rs/dirs/3.0.0/dirs/fn.cache_dir.html
+/// The download artifacts are saved in the user's [cache directory](
+/// https://docs.rs/dirs/3.0.0/dirs/fn.cache_dir.html).
 ///
 /// Downloads a single zip file for all components, as used in CLDR 38 and later.
 ///
@@ -34,13 +34,13 @@ use std::path::PathBuf;
 ///     use std::convert::TryFrom;
 ///     use icu_provider::prelude::*;
 ///
-///     let paths = downloader.download()
+///     let paths = downloader.download(None)
 ///         .expect("The data should download successfully");
 ///
 ///     let data_provider = PluralsProvider::try_from(&paths as &dyn CldrPaths)
 ///         .expect("The data should be well-formed after downloading");
 ///
-///     let data: DataPayload<icu_plurals::provider::PluralRuleStringsV1Marker> = data_provider
+///     let data: DataPayload<icu_plurals::provider::PluralRulesV1Marker> = data_provider
 ///         .load_payload(&DataRequest {
 ///             resource_path: ResourcePath {
 ///                 key: icu_plurals::provider::key::ORDINAL_V1,
@@ -53,7 +53,9 @@ use std::path::PathBuf;
 ///         .unwrap()
 ///         .take_payload()
 ///         .unwrap();
-///     assert_eq!(data.get().few, Some(Cow::Borrowed("n % 10 = 3 and n % 100 != 13")));
+///     let rule = "n % 10 = 3 and n % 100 != 13".parse()
+///         .expect("Failed to parse plural rule");
+///     assert_eq!(data.get().few, Some(rule));
 /// }
 ///
 /// // Calling demo(downloader) will cause the data to actually get downloaded.
@@ -77,8 +79,8 @@ impl CldrAllInOneDownloader {
     ///
     /// Arguments:
     ///
-    /// - `github_tag`: a tag in the CLDR JSON repositories, such as "38.1.0":
-    ///   https://github.com/unicode-cldr/cldr-json/tags
+    /// - `github_tag`: a [tag in the CLDR JSON repositories](https://github.com/unicode-cldr/cldr-json/tags),
+    ///    such as "38.1.0"
     /// - `locale_subset`: either "modern" (fewer locales, smaller download) or "full" (more
     ///   locales, larger download)
     pub fn try_new_from_github(github_tag: &str, locale_subset: &str) -> Result<Self, Error> {
@@ -95,12 +97,13 @@ impl CldrAllInOneDownloader {
         })
     }
 
-    pub fn download(self) -> Result<CldrPathsAllInOne, Error> {
+    pub fn download(self, uprops_root: Option<PathBuf>) -> Result<CldrPathsAllInOne, Error> {
         // TODO(#297): Implement this async.
         let downloaded = io_util::download_and_unzip(&self.url, &self.cache_dir)?;
         Ok(CldrPathsAllInOne {
             cldr_json_root: downloaded,
             locale_subset: self.locale_subset,
+            uprops_root,
         })
     }
 }
