@@ -4,6 +4,7 @@
 
 use displaydoc::Display;
 use icu_locid::LanguageIdentifier;
+use icu_provider::DataError;
 use std::path::{Path, PathBuf};
 
 #[cfg(feature = "download")]
@@ -23,8 +24,6 @@ pub enum Error {
     #[cfg(feature = "download")]
     #[displaydoc("{0}")]
     Download(download::Error),
-    #[displaydoc("poisoned lock on CLDR provider")]
-    Poison,
 }
 
 impl std::error::Error for Error {}
@@ -84,3 +83,28 @@ impl From<MissingSourceError> for Error {
         Self::MissingSource(err)
     }
 }
+
+impl From<Error> for DataError {
+    fn from(err: Error) -> Self {
+        use Error::*;
+        match err {
+            Io(e, Some(path_buf)) => DataError::from(e).with_path(&path_buf),
+            Io(e, None) => DataError::from(e),
+            Json(e, Some(path_buf)) => DataError::custom("CLDR JSON: Json Parse Error")
+                .with_error_context(&e)
+                .with_path(&path_buf),
+            Json(e, None) => {
+                DataError::custom("CLDR JSON: Json Parse Error").with_error_context(&e)
+            }
+            Custom(s, Some(langid)) => DataError::custom("CLDR JSON: Custom")
+                .with_display_context(&s)
+                .with_display_context(&langid),
+            Custom(s, None) => DataError::custom("CLDR JSON: Custom").with_display_context(&s),
+            MissingSource(e) => {
+                DataError::custom("CLDR JSON: MissingSource").with_error_context(&e)
+            }
+            #[cfg(feature = "download")]
+            Download(e) => DataError::custom("CLDR JSON: Download").with_error_context(&e),
+        }
+    }
+}
