@@ -2,20 +2,20 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+use crate::cldr_serde;
 use crate::error::Error;
 use crate::reader::{get_langid_subdirectories, open_reader};
+use crate::support::KeyedDataProvider;
 use crate::CldrPaths;
 use icu_datetime::provider::{key, time_zones::*};
 use icu_locid::LanguageIdentifier;
-use icu_provider::{
-    iter::{IterableDataProviderCore, KeyedDataProvider},
-    prelude::*,
-};
+use icu_provider::iter::IterableProvider;
+use icu_provider::prelude::*;
+use litemap::LiteMap;
 
 use std::convert::TryFrom;
-use std::marker::PhantomData;
 
-mod cldr_json;
+mod convert;
 
 /// All keys that this module is able to produce.
 pub const ALL_KEYS: [ResourceKey; 6] = [
@@ -29,57 +29,58 @@ pub const ALL_KEYS: [ResourceKey; 6] = [
 
 /// A data provider reading from CLDR JSON zones files.
 #[derive(PartialEq, Debug)]
-pub struct TimeZonesProvider<'data> {
-    data: Vec<(LanguageIdentifier, cldr_json::LangTimeZones)>,
-    phantom: PhantomData<&'data ()>, // placeholder for when we need the lifetime param
+pub struct TimeZonesProvider {
+    data: LiteMap<LanguageIdentifier, cldr_serde::time_zone_names::LangTimeZones>,
 }
 
-impl TryFrom<&dyn CldrPaths> for TimeZonesProvider<'_> {
+impl TryFrom<&dyn CldrPaths> for TimeZonesProvider {
     type Error = Error;
     fn try_from(cldr_paths: &dyn CldrPaths) -> Result<Self, Self::Error> {
-        let mut data = vec![];
+        let mut data = LiteMap::new();
 
-        let path = cldr_paths.cldr_dates()?.join("main");
+        let path = cldr_paths.cldr_dates("gregory")?.join("main");
 
         let locale_dirs = get_langid_subdirectories(&path)?;
 
         for dir in locale_dirs {
             let path = dir.join("timeZoneNames.json");
 
-            let mut resource: cldr_json::Resource =
+            let resource: cldr_serde::time_zone_names::Resource =
                 serde_json::from_reader(open_reader(&path)?).map_err(|e| (e, path))?;
-            data.append(&mut resource.main.0);
+            data.extend_from_litemap(resource.main.0);
         }
 
-        Ok(Self {
-            data,
-            phantom: PhantomData,
-        })
+        Ok(Self { data })
     }
 }
 
-impl TryFrom<&str> for TimeZonesProvider<'_> {
+impl TryFrom<&str> for TimeZonesProvider {
     type Error = Error;
     fn try_from(input: &str) -> Result<Self, Self::Error> {
-        let resource: cldr_json::Resource =
+        let resource: cldr_serde::time_zone_names::Resource =
             serde_json::from_str(input).map_err(|e| Error::Json(e, None))?;
         Ok(Self {
             data: resource.main.0,
-            phantom: PhantomData,
         })
     }
 }
 
-impl<'data> KeyedDataProvider for TimeZonesProvider<'data> {
+impl KeyedDataProvider for TimeZonesProvider {
     fn supports_key(resc_key: &ResourceKey) -> Result<(), DataError> {
-        if resc_key.category != ResourceCategory::TimeZone || resc_key.version != 1 {
-            return Err(resc_key.into());
+        // TODO(#442): Clean up KeyedDataProvider
+        match *resc_key {
+            key::TIMEZONE_FORMATS_V1 => Ok(()),
+            key::TIMEZONE_EXEMPLAR_CITIES_V1 => Ok(()),
+            key::TIMEZONE_GENERIC_NAMES_LONG_V1 => Ok(()),
+            key::TIMEZONE_GENERIC_NAMES_SHORT_V1 => Ok(()),
+            key::TIMEZONE_SPECIFIC_NAMES_LONG_V1 => Ok(()),
+            key::TIMEZONE_SPECIFIC_NAMES_SHORT_V1 => Ok(()),
+            _ => Err(DataErrorKind::MissingResourceKey.with_key(*resc_key)),
         }
-        Ok(())
     }
 }
 
-impl<'data> IterableDataProviderCore for TimeZonesProvider<'data> {
+impl IterableProvider for TimeZonesProvider {
     #[allow(clippy::needless_collect)] // https://github.com/rust-lang/rust-clippy/issues/7526
     fn supported_options_for_key(
         &self,
@@ -98,22 +99,19 @@ impl<'data> IterableDataProviderCore for TimeZonesProvider<'data> {
 }
 
 macro_rules! impl_data_provider {
-    ($id:ident: $lt:lifetime, $marker:ident) => {
-        impl<$lt> DataProvider<$lt, $marker> for TimeZonesProvider<$lt> {
-            fn load_payload(
-                &self,
-                req: &DataRequest,
-            ) -> Result<DataResponse<$lt, $marker>, DataError> {
+    ($id:ident, $marker:ident) => {
+        impl DataProvider<$marker> for TimeZonesProvider {
+            fn load_payload(&self, req: &DataRequest) -> Result<DataResponse<$marker>, DataError> {
                 TimeZonesProvider::supports_key(&req.resource_path.key)?;
                 let langid = req.try_langid()?;
-                let time_zones = match self.data.binary_search_by_key(&langid, |(lid, _)| lid) {
-                    Ok(idx) => &self.data[idx].1.dates.time_zone_names,
-                    Err(_) => return Err(DataError::MissingResourceOptions(req.clone())),
+                let time_zones = match self.data.get(&langid) {
+                    Some(v) => &v.dates.time_zone_names,
+                    None => return Err(DataErrorKind::MissingLocale.with_req(req)),
                 };
+                let metadata = DataResponseMetadata::default();
+                // TODO(#1109): Set metadata.data_langid correctly.
                 Ok(DataResponse {
-                    metadata: DataResponseMetadata {
-                        data_langid: req.resource_path.options.langid.clone(),
-                    },
+                    metadata,
                     payload: Some(DataPayload::from_owned($id::from(time_zones.clone()))),
                 })
             }
@@ -121,30 +119,30 @@ macro_rules! impl_data_provider {
     };
 }
 
-icu_provider::impl_dyn_provider!(TimeZonesProvider<'data>, {
+icu_provider::impl_dyn_provider!(TimeZonesProvider, {
     key::TIMEZONE_FORMATS_V1 => TimeZoneFormatsV1Marker,
     key::TIMEZONE_EXEMPLAR_CITIES_V1 => ExemplarCitiesV1Marker,
     key::TIMEZONE_GENERIC_NAMES_LONG_V1 => MetaZoneGenericNamesLongV1Marker,
     key::TIMEZONE_GENERIC_NAMES_SHORT_V1 => MetaZoneGenericNamesShortV1Marker,
     key::TIMEZONE_SPECIFIC_NAMES_LONG_V1 => MetaZoneSpecificNamesLongV1Marker,
     key::TIMEZONE_SPECIFIC_NAMES_SHORT_V1 => MetaZoneSpecificNamesShortV1Marker,
-}, ERASED);
+}, SERDE_SE);
 
-icu_provider::impl_dyn_provider!(TimeZonesProvider<'data>, {
-    key::TIMEZONE_FORMATS_V1 => TimeZoneFormatsV1Marker,
-    key::TIMEZONE_EXEMPLAR_CITIES_V1 => ExemplarCitiesV1Marker,
-    key::TIMEZONE_GENERIC_NAMES_LONG_V1 => MetaZoneGenericNamesLongV1Marker,
-    key::TIMEZONE_GENERIC_NAMES_SHORT_V1 => MetaZoneGenericNamesShortV1Marker,
-    key::TIMEZONE_SPECIFIC_NAMES_LONG_V1 => MetaZoneSpecificNamesLongV1Marker,
-    key::TIMEZONE_SPECIFIC_NAMES_SHORT_V1 => MetaZoneSpecificNamesShortV1Marker,
-}, SERDE_SE, 'data);
-
-impl_data_provider!(TimeZoneFormatsV1: 'd, TimeZoneFormatsV1Marker);
-impl_data_provider!(ExemplarCitiesV1: 'd, ExemplarCitiesV1Marker);
-impl_data_provider!(MetaZoneGenericNamesLongV1: 'd, MetaZoneGenericNamesLongV1Marker);
-impl_data_provider!(MetaZoneGenericNamesShortV1: 'd, MetaZoneGenericNamesShortV1Marker);
-impl_data_provider!(MetaZoneSpecificNamesLongV1: 'd, MetaZoneSpecificNamesLongV1Marker);
-impl_data_provider!(MetaZoneSpecificNamesShortV1: 'd, MetaZoneSpecificNamesShortV1Marker);
+impl_data_provider!(TimeZoneFormatsV1, TimeZoneFormatsV1Marker);
+impl_data_provider!(ExemplarCitiesV1, ExemplarCitiesV1Marker);
+impl_data_provider!(MetaZoneGenericNamesLongV1, MetaZoneGenericNamesLongV1Marker);
+impl_data_provider!(
+    MetaZoneGenericNamesShortV1,
+    MetaZoneGenericNamesShortV1Marker
+);
+impl_data_provider!(
+    MetaZoneSpecificNamesLongV1,
+    MetaZoneSpecificNamesLongV1Marker
+);
+impl_data_provider!(
+    MetaZoneSpecificNamesShortV1,
+    MetaZoneSpecificNamesShortV1Marker
+);
 
 #[cfg(test)]
 mod tests {
